#include "OgreDepthShadowRecieverRenderTechnique.h"
#include "OgreTechniqueGroup.h"
#include "OgreIlluminationManager.h"
#include "OgreDepthShadowMapRenderingRun.h"

OgreDepthShadowRecieverRenderTechnique::OgreDepthShadowRecieverRenderTechnique(
												int maxlights,
												String shadowVertexProgram,
												String shadowFragmentProgram,
												Pass* pass,
												OgreRenderable* parentRenderable,
												OgreTechniqueGroup* parentTechniqueGroup)
							:RenderTechnique( parentRenderable, parentTechniqueGroup),
							OgreRenderTechnique(pass, parentRenderable, parentTechniqueGroup),
							DepthShadowRecieverRenderTechnique(parentRenderable, parentTechniqueGroup)
{
	this->maxlights = maxlights;
	this->shadowVertexProgram = shadowVertexProgram ;
	this->shadowFragmentProgram = shadowFragmentProgram;

		
	//insert new passes
	Ogre::Technique* techn = pass->getParent();
	Technique::PassIterator it = techn->getPassIterator();
	
	int index = 0;
	while(it.hasMoreElements())
	{
		if( it.getNext() == pass)
			break;
		index++;
		it.moveNext();
	}
	
	index++;
	for(int i = 0; i < maxlights; i++)
	{
		int lastpass = techn->getNumPasses();
		Pass* newpass = techn->createPass();
		passes.push_back(newpass);

		newpass->setVertexProgram(shadowVertexProgram);
		newpass->setFragmentProgram(shadowFragmentProgram);
		
		GpuProgramParameters* Vparams = newpass->getVertexProgramParameters().getPointer();
		Vparams->setNamedAutoConstant("worldViewProj",
									GpuProgramParameters::ACT_WORLDVIEWPROJ_MATRIX);
		Vparams->setNamedAutoConstant("world",
									GpuProgramParameters::ACT_WORLD_MATRIX);
		GpuProgramParameters* Fparams = newpass->getFragmentProgramParameters().getPointer();
		
		TextureUnitState* st = newpass->createTextureUnitState();		
		st->setTextureFiltering(TFO_BILINEAR);		
		st = newpass->createTextureUnitState();		
		st->setTextureFiltering(TFO_BILINEAR);		

		newpass->setSceneBlending(SBT_MODULATE);
		newpass->setDepthBias(7);
		techn->movePass(lastpass, index);			
	}
	
}

OgreDepthShadowRecieverRenderTechnique::~OgreDepthShadowRecieverRenderTechnique()
{

}

void OgreDepthShadowRecieverRenderTechnique::update(unsigned long frameNum)
{
	LightList lights;
	Root::getSingleton()._getCurrentSceneManager()->_populateLightList(
						((OgreSharedRuns*) sharedRuns)->getRootPosition(),
							100000.0,
							lights);
	
	//fill passes
	for(unsigned int i = 0; i < passes.size(); i++)
	{
		if(lights.size() > i)
		{
			//create run if not exists
			OgreIlluminationManager::getSingleton().createPerLightRun(lights.at(i)->getName(),
																		ILLUMRUN_DEPTH_SHADOWMAP);
			//update light depth map
			OgreIlluminationManager::getSingleton().updatePerLightRun(lights.at(i)->getName(),
																		ILLUMRUN_DEPTH_SHADOWMAP,
																		frameNum);
			
			//set texture to pass
			OgreDepthShadowMapRenderingRun* depthRun = 
				(OgreDepthShadowMapRenderingRun*) OgreIlluminationManager::getSingleton()
													.getPerLightRun(lights.at(i)->getName(), 
													ILLUMRUN_DEPTH_SHADOWMAP)->asOgreRenderingRun();
					
			passes.at(i)->getTextureUnitState(0)->setTextureName(
				depthRun->getDepthMapTextureName(0));
			passes.at(i)->getTextureUnitState(1)->setTextureName(
				depthRun->getDepthMapTextureName(1));

			//TODO: set matrices
			GpuProgramParametersSharedPtr fpParams = passes.at(i)->getFragmentProgramParameters();
			fpParams->setNamedConstant("lightView", depthRun->getLightViewMatrix(0)); 
			fpParams->setNamedConstant("lightViewProj", depthRun->getLightViewProjMatrix(0)); 
		//	fpParams->setNamedConstant("lightView2", depthRun->getLightViewMatrix(1)); 
		//	fpParams->setNamedConstant("lightViewProj2", depthRun->getLightViewProjMatrix(1)); 
			
			passes.at(i)->setActive(true);
		}	
		else
			passes.at(i)->setActive(false);

	}
}


///Technique parsers
namespace DepthShadowRecieverParsers
{
	void parseMaxLights(String& params, RenderTechniqueFactory* factory)
	{
		OgreDepthShadowRecieverRenderTechniqueFactory* f = (OgreDepthShadowRecieverRenderTechniqueFactory*) factory;
		f->maxlights =  StringConverter::parseInt(params);
	}

	void parseVertexProgram(String& params, RenderTechniqueFactory* factory)
	{
		OgreDepthShadowRecieverRenderTechniqueFactory* f = (OgreDepthShadowRecieverRenderTechniqueFactory*) factory;
		f->shadowVertexProgram =  params;
	}

	void parseFragmentProgram(String& params, RenderTechniqueFactory* factory)
	{
		OgreDepthShadowRecieverRenderTechniqueFactory* f = (OgreDepthShadowRecieverRenderTechniqueFactory*) factory;
		f->shadowFragmentProgram =  params;
	}
}

OgreDepthShadowRecieverRenderTechniqueFactory::OgreDepthShadowRecieverRenderTechniqueFactory()
{
	typeName = "DepthShadowReciever";

	using namespace DepthShadowRecieverParsers;
	//register parsers
	this->attributeParsers.insert(AttribParserList::value_type("max_light_count", (ILLUM_ATTRIBUTE_PARSER) parseMaxLights));
	this->attributeParsers.insert(AttribParserList::value_type("vertex_program_name", (ILLUM_ATTRIBUTE_PARSER) parseVertexProgram));
	this->attributeParsers.insert(AttribParserList::value_type("fragment_program_name", (ILLUM_ATTRIBUTE_PARSER) parseFragmentProgram));
}

OgreRenderTechnique* OgreDepthShadowRecieverRenderTechniqueFactory::createInstance( 
										IllumTechniqueParams* params,
										Pass* pass,
										OgreRenderable* parentRenderable,
										OgreTechniqueGroup* parentTechniqueGroup)
{	
	//reset parameters
	maxlights = 1;
	shadowVertexProgram = "GameTools/ShadowMap/ShadowVS";
	shadowFragmentProgram = "GameTools/ShadowMap/ShadowPS";

	parseParams(params);

	OgreDepthShadowRecieverRenderTechnique* result = new OgreDepthShadowRecieverRenderTechnique(
												maxlights,
												shadowVertexProgram,
												shadowFragmentProgram,
												pass,
												parentRenderable,
												parentTechniqueGroup);	

	return result;
}

